
### Style Guides

#### POLICY

You `SHOULD` adhere to established style guides for programming languages, where possible.

You `MUST` adhere to the most basic rules defined by the
[EditorConfig](http://editorconfig.org/)
[file provided with the Technical Reference source repository](https://gitlab.gwdg.de/fe/technical-reference/-/raw/main/.editorconfig).

#### Why Use Style Guides

To put it simply, style guides improve your code's readability, maintainability, and portability. Your code will be much
more understandable to others (and yourself, returning to your code after a while) if it is written in a coherent style.
A uniform style will improve code quality by facilitating debugging and review processes. It might even help you avoid
bugs in the first place by avoiding unnecessary complexity. Finally, handing over a project will be much easier when
the new as well as the old developer can quickly comprehend the code base.

#### General

Style encompasses the style of your code in the narrow sense (your functions, classes, variables, etc.), as well as that
of your documentation. Many of the style guides we use give directions on how to write code and documentation.
You `SHOULD` name and link the style guide you use in your `README`.

#### Recommendations

If your project already has a style guide, stick to it, until there are good reasons to change.
If you are looking for a style guide for a new language or project here are some recommendations:

- **Java**: [Google style guide for Java](https://github.com/google/styleguide)
- **JavaScript**: [Airbnb JavaScript Style Guide](https://github.com/airbnb/javascript).
- **HTML/CSS**: [Google HTML/CSS Style Guide](https://google.github.io/styleguide/htmlcssguide.html).
- **XQuery**: [xqdoc style guide](http://xqdoc.org/xquery-style.pdf)
- **XSLT**: There is no official style guide for XSLT, we wrote
[our own](https://gitlab.gwdg.de/fe/technical-reference/tree/main/style-guides/rdd-xslt.md)
- **Python**: [PEP 8](https://www.python.org/dev/peps/pep-0008/) or
[Django-Styleguide](https://docs.djangoproject.com/en/dev/internals/contributing/writing-code/coding-style/)
- **SPARQL**: There is no official style guide, so we have some advice how to format and use SPARQL code
[here](https://gitlab.gwdg.de/fe/technical-reference/tree/main/style-guides/rdd-sparql.md).
- **Golang**: [Google Go style guide](https://google.github.io/styleguide/go/guide)
